using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;
using Hardcodet.Scheduling;
using NUnit.Framework;


namespace Scheduling_Tests
{
  /// <summary>
  /// 
  /// </summary>
  [TestFixture]
  public class Given_Job_If_System_Time_Is_Being_Changed : SchedulerTestBase
  {


    [Test]
    public void Previously_Planned_Jobs_Should_Not_Execute_Yet_If_Time_Is_Set_To_Earlier_Value()
    {
      //freeze time
      var now = DateTimeOffset.Now;
      SystemTime.Now = () => now;
      DateTimeOffset start = now.AddMilliseconds(300);

      //run self-test every 200 ms
      Scheduler.SelfTestInterval = 200;

      bool invoked = false;

      //submit the job - it should start in 300ms
      Job job = new Job();
      job.Run.From(start).Once();

      Scheduler.SubmitJob(job, j => invoked = true);

      //make it earlier in time - the job is now 1300ms away
      SystemTime.Now = () => now.AddMilliseconds(-1000);

      //make sure the job didn't execute
      Thread.Sleep(2000);
      Assert.IsFalse(invoked);

      //let time run as usual again
      SystemTime.Reset();
      Thread.Sleep(1500);
      Assert.IsTrue(invoked);
    }


    [Test]
    public void Setting_Time_To_Later_Value_Will_Trigger_Pending_Jobs_On_Next_Self_Test()
    {
      //freeze time
      var now = DateTimeOffset.Now;
      SystemTime.Now = () => now;
      DateTimeOffset start = DateTime.Now.AddHours(2);

      //run self-test every 150 ms
      Scheduler.SelfTestInterval = 150;

      bool invoked = false;

      Job<int> job = new Job<int>();
      job.Run.From(start).Once();
      Scheduler.SubmitJob(job, j => invoked = true);

      Thread.Sleep(500);
      Assert.IsFalse(invoked);

      //make the job pending
      SystemTime.Now = () => job.StartTime.AddMilliseconds(1);
      Thread.Sleep(200);

      Assert.IsTrue(invoked);

    }
  }
}
