using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;
using Hardcodet.Scheduling;
using NUnit.Framework;


namespace Scheduling_Tests
{
  /// <summary>
  /// 
  /// </summary>
  [TestFixture]
  public class Given_Job_When_Execution_Causes_Exception : SchedulerTestBase
  {


    [Test]
    public void Unhandled_Exception_Should_Not_Cause_Scheduler_To_Fail()
    {
      int exceptions = 0, success = 0;
      Job<int> job = new Job<int>();
      job.Run.Times(7).Every.Milliseconds(150);

      Scheduler.SubmitJob(job, (j, i) =>
      {
        j.Data++;
        if(i % 2 == 0)
        {
          exceptions++;
          throw new InvalidOperationException("Exception on worker thread");
        }
        success++;
      });

      Thread.Sleep(1200);
      Assert.AreEqual(4, exceptions);
      Assert.AreEqual(3, success);
    }



    [Test]
    public void Unhandled_Exception_Should_Be_Reported_To_Scheduler_Handler_If_Registered()
    {
      Job j1 = new Job("j1");
      Job j2 = new Job("j2");

      j1.Run.Every.Milliseconds(150).Times(4);
      j2.Run.Every.Milliseconds(200).Times(3);

      int count1 = 0, count2 = 0, exceptions = 0;

      Scheduler.JobExceptionHandler = (j, e) =>
                                        {
                                          exceptions++;
                                          Assert.AreSame(j1, j);
                                          Assert.IsInstanceOf<InvalidOperationException>(e);
                                        };


      Scheduler.SubmitJob(j1, j =>
      {
        count1++;
        if(count1 % 2 == 1) throw new InvalidOperationException("J1 failed");
      });

      Scheduler.SubmitJob(j2, j =>
      {
        count2++;
      });

      Thread.Sleep(1000);
      Assert.AreEqual(2, exceptions);
      Assert.AreEqual(4, count1);
      Assert.AreEqual(3, count2);
    }
  }
}
