using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;
using Hardcodet.Scheduling;
using NUnit.Framework;


namespace Scheduling_Tests
{
  /// <summary>
  /// 
  /// </summary>
  [TestFixture]
  public class Given_Job_When_Resuming_Execution : SchedulerTestBase
  {
    private Job job;
    private int counter;
    protected override void InitInternal()
    {
      counter = 0;
      job = new Job("abc");
      Scheduler.MinJobInterval = 1;

      //if there is no interval, we cannot reschedule
      job.Run.Every.Milliseconds(100).Until(DateTime.Now.AddMilliseconds(500));
      Scheduler.SubmitJob(job, j => counter++);
    }


    [Test]
    public void Resuming_Should_Continue_Execution()
    {
      Assert.IsTrue(job.Pause());
      Assert.AreEqual(JobState.Paused, job.State);

      Thread.Sleep(150);
      Assert.AreEqual(0, counter);

      Assert.IsTrue(job.Resume());
      Assert.AreEqual(JobState.Active, job.State);

      Thread.Sleep(310);
      Assert.AreEqual(3, counter);
    }


    [Test]
    public void Resuming_Via_Scheduler_Should_Continue_Execution()
    {
      Assert.IsTrue(job.Pause());
      Assert.AreEqual(JobState.Paused, job.State);

      Thread.Sleep(150);
      Assert.AreEqual(0, counter);

      Assert.IsTrue(Scheduler.ResumeJob(job.JobId));
      Assert.AreEqual(JobState.Active, job.State);

      Thread.Sleep(310);
      Assert.AreEqual(3, counter);
    }


    [Test]
    public void Resuming_Should_Not_Work_For_Active_Jobs()
    {
      Assert.IsFalse(job.Resume());
      Assert.AreEqual(JobState.Active, job.State);
    }


    [Test]
    public void Resuming_Should_Not_Work_For_Finished_Jobs ()
    {
      Thread.Sleep(550);

      Assert.AreEqual(JobState.Finished, job.State);
      Assert.IsFalse(job.Resume());
      Assert.AreEqual(JobState.Finished, job.State);
    }

    [Test]
    public void Resuming_Should_Not_Work_For_Canceled_Jobs()
    {
      job.Cancel();

      Assert.AreEqual(JobState.Canceled, job.State);
      Assert.IsFalse(job.Resume());
      Assert.AreEqual(JobState.Canceled, job.State);
    }

    
  }
}