using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;
using Hardcodet.Scheduling;
using NUnit.Framework;


namespace Scheduling_Tests
{
  /// <summary>
  /// 
  /// </summary>
  [TestFixture]
  public class Given_Scheduler_When_Cancelling_Jobs : SchedulerTestBase
  {
    private List<Job> jobs;
    private DateTimeOffset now;

    protected override void InitInternal()
    {
      now = DateTimeOffset.Now;
      SystemTime.Now = () => now;
      jobs = new List<Job>();

      for (int i = 0; i < 5; i++)
      {
        Job job = new Job(i.ToString());
        job.Run.From(now.AddMinutes(i + 0.5)).Once();
        Scheduler.SubmitJob(job, j => { });
        jobs.Add(job);
      }
    }

    


    [Test]
    public void Cancelled_Job_Should_Not_Execute()
    {
      SystemTime.Reset();
      var job = new Job();
      job.Run.From(now.AddMilliseconds(100)).Once();

      bool invoked = false;
      Scheduler.SubmitJob(job, j => invoked = true);
      
      Thread.Sleep(50);
      Assert.IsFalse(invoked);
      Scheduler.CancelJob(job.JobId);

      Thread.Sleep(100);
      Assert.IsFalse(invoked);
    }



    [Test]
    public void Cancelling_Job_Via_Scheduler_Should_Change_Job_State()
    {
      SystemTime.Reset();
      var job = new Job();
      job.Run.From(now.AddMilliseconds(100)).Once();

      bool invoked = false;
      Scheduler.SubmitJob(job, j => invoked = true);

      Thread.Sleep(50);
      Scheduler.CancelJob(job.JobId);

      Assert.AreEqual(JobState.Canceled, job.State);
      Thread.Sleep(100);
      Assert.IsFalse(invoked);

    }



    [Test]
    public void Removing_Last_Remaining_Job_Should_Disable_Timer()
    {
      for (int i = 0; i < jobs.Count; i++)
      {
        var job = jobs[i];
        Assert.NotNull(Scheduler.NextExecution);
        Assert.IsTrue(Scheduler.CancelJob(job.JobId));
      }

      Assert.IsNull(Scheduler.NextExecution);
    }

    [Test]
    public void Removing_Next_Pending_Job_Should_Cause_Rescheduling()
    {
      Assert.AreEqual(jobs[0].StartTime, Scheduler.NextExecution);
      Scheduler.CancelJob(jobs[0].JobId);
      Assert.AreNotEqual(jobs[0].StartTime, Scheduler.NextExecution);
      Assert.AreEqual(jobs[1].StartTime, Scheduler.NextExecution);
    }

    [Test]
    public void Removing_Non_Pending_Job_Should_Not_Cause_Rescheduling()
    {
      Assert.AreEqual(jobs[0].StartTime, Scheduler.NextExecution);
      Scheduler.CancelJob(jobs[4].JobId);
      Assert.AreEqual(jobs[0].StartTime, Scheduler.NextExecution);
    }
  }

}
