using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Xml;
using Hardcodet.Scheduling;
using NUnit.Framework;


namespace Scheduling_Tests
{
  /// <summary>
  /// 
  /// </summary>
  [TestFixture]
  public class Given_Scheduler_When_Self_Testing : SchedulerTestBase
  {
    private DateTimeOffset now;

    protected override void InitInternal()
    {
      //freeze time
      now = DateTimeOffset.Now;
      SystemTime.Now = () => now;
    }


    [Test]
    public void Removing_Last_Job_Should_Stop_Self_Testing_Timer()
    {
      bool invoked;
      Job job = new Job("abc");
      job.Run.From(now.AddHours(2)).Once();

      Assert.IsNull(Scheduler.NextExecution);

      Scheduler.SubmitJob(job, j => invoked = true);

      Assert.IsNotNull(Scheduler.NextExecution);

      Scheduler.CancelJob(job.JobId);

      Assert.IsNull(Scheduler.NextExecution);
    }


    [Test]
    public void Adding_Job_That_Starts_Late_Should_Cause_System_To_Run_After_Self_Test_Interval()
    {
      bool invoked;
      Job job = new Job("abc");
      job.Run.From(now.AddHours(2)).Once();


      //submitting job causes to run at next self test      
      Scheduler.SubmitJob(job, j => invoked = true);
      Assert.AreEqual(now.AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);
    }


    [Test]
    public void Changing_System_Time_To_Later_Should_Correct_Timer_After_Self_Test_Interval()
    {
      var now = DateTimeOffset.Now;
      SystemTime.Now = () => now;

      Job job = new Job("abc");
      job.Run.From(now.AddHours(2)).Once();

      Scheduler.SubmitJob(job, j => { });

      Scheduler.SelfTestInterval = 100;
      Assert.AreEqual(now.AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);

      //its later
      SystemTime.Now = () => now.AddMinutes(10);

      //next execution rescheduled properly?
      Thread.Sleep(150);
      Assert.AreEqual(now.AddMinutes(10).AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);
    }


    [Test]
    public void Changing_Self_Test_Timer_Should_Reschedule()
    {
      var now = DateTimeOffset.Now;
      SystemTime.Now = () => now;

      Job job = new Job("abc");
      job.Run.From(now.AddHours(2)).Once();

      Scheduler.SubmitJob(job, j => { });
      Assert.AreEqual(now.AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);

      //go lower
      Scheduler.SelfTestInterval = 1000;
      Assert.AreEqual(now.AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);

      //go higher
      Scheduler.SelfTestInterval = 3000;
      Assert.AreEqual(now.AddMilliseconds(Scheduler.SelfTestInterval), Scheduler.NextExecution);
    }

  }
}
